package cn.com.infosec.util.encoders;

import java.io.OutputStream;

public class Base64Encoder implements Encoder {
   protected final byte[] encodingTable = new byte[]{65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 43, 47};
   protected byte padding = 61;
   protected final byte[] decodingTable = new byte[128];

   public Base64Encoder() {
      this.initialiseDecodingTable();
   }

   private boolean ignore(char c) {
      return c == '\n' || c == '\r' || c == '\t' || c == ' ';
   }

   private int nextI(byte[] data, int i, int finish) {
      while(i < finish && this.ignore((char)data[i])) {
         ++i;
      }

      return i;
   }

   private int decodeLastBlock(OutputStream out, char c1, char c2, char c3, char c4) {
      byte var6;
      byte[] var7;
      byte var8;
      byte var12;
      if (c3 == (var6 = this.padding)) {
         var12 = (var7 = this.decodingTable)[c1];
         var8 = var7[c2];
         out.write(var12 << 2 | var8 >> 4);
         return 1;
      } else {
         byte c11;
         byte c21;
         byte[] var10001;
         if (c4 == var6) {
            var10001 = var7 = this.decodingTable;
            c11 = var7[c1];
            var12 = c21 = var10001[c2];
            var8 = var7[c3];
            out.write(c11 << 2 | c21 >> 4);
            out.write(var12 << 4 | var8 >> 2);
            return 2;
         } else {
            var10001 = var7 = this.decodingTable;
            c11 = var7[c1];
            c21 = var7[c2];
            byte c31;
            var12 = c31 = var10001[c3];
            var8 = var7[c4];
            out.write(c11 << 2 | c21 >> 4);
            out.write(c21 << 4 | c31 >> 2);
            out.write(var12 << 6 | var8);
            return 3;
         }
      }
   }

   private int nextI(String data, int i, int finish) {
      while(i < finish && this.ignore(data.charAt(i))) {
         ++i;
      }

      return i;
   }

   protected void initialiseDecodingTable() {
      byte[] var2;
      for(int var1 = 0; var1 < (var2 = this.encodingTable).length; ++var1) {
         byte var3 = var2[var1];
         this.decodingTable[var3] = (byte)var1;
      }

   }

   public int encode(byte[] data, int off, int length, OutputStream out) {
      int var5 = length - (length %= 3);

      int var7;
      for(int var6 = off; var6 < (var7 = off + var5); var6 += 3) {
         var7 = data[var6] & 255;
         int var8 = data[var6 + 1] & 255;
         int var9 = data[var6 + 2] & 255;
         out.write(this.encodingTable[var7 >>> 2 & 63]);
         out.write(this.encodingTable[(var7 << 4 | var8 >>> 4) & 63]);
         out.write(this.encodingTable[(var8 << 2 | var9 >>> 6) & 63]);
         out.write(this.encodingTable[var9 & 63]);
      }

      int var10;
      int data1;
      Base64Encoder var10001;
      Base64Encoder var10003;
      Base64Encoder var10005;
      Base64Encoder var10007;
      int var10008;
      if (length != 1) {
         if (length == 2) {
            var10001 = this;
            var10003 = this;
            var10005 = this;
            var10007 = this;
            var10 = data[var7] & 255;
            var10008 = data[var7 + 1] & 255;
            int var10010 = var10;
            var10 = var10 >>> 2 & 63;
            data1 = (var10010 << 4 | var10008 >>> 4) & 63;
            off = var10008 << 2 & 63;
            out.write(var10007.encodingTable[var10]);
            out.write(var10005.encodingTable[data1]);
            out.write(var10003.encodingTable[off]);
            out.write(var10001.padding);
         }
      } else {
         var10001 = this;
         var10003 = this;
         var10005 = this;
         var10007 = this;
         var10008 = data[var7] & 255;
         var10 = (data[var7] & 255) >>> 2 & 63;
         data1 = var10008 << 4 & 63;
         out.write(var10007.encodingTable[var10]);
         out.write(var10005.encodingTable[data1]);
         out.write(var10003.padding);
         out.write(var10001.padding);
      }

      var10 = var5 / 3 * 4;
      byte data2;
      if (length == 0) {
         data2 = 0;
      } else {
         data2 = 4;
      }

      return var10 + data2;
   }

   public int decode(byte[] data, int off, int length, OutputStream out) {
      int var5 = 0;

      for(length += off; length > off && this.ignore((char)data[length - 1]); --length) {
      }

      int var6 = length - 4;

      for(off = this.nextI(data, off, var6); off < var6; off = this.nextI(data, off, var6)) {
         byte var7 = this.decodingTable[data[off++]];
         off = this.nextI(data, off, var6);
         byte var8 = this.decodingTable[data[off++]];
         off = this.nextI(data, off, var6);
         byte var9;
         byte var10003 = var9 = this.decodingTable[data[off++]];
         byte var10007 = var7;
         off = this.nextI(data, off, var6);
         var7 = this.decodingTable[data[off++]];
         out.write(var10007 << 2 | var8 >> 4);
         out.write(var8 << 4 | var9 >> 2);
         out.write(var10003 << 6 | var7);
         var5 += 3;
      }

      char var10 = (char)data[var6];
      char data1 = (char)data[length - 3];
      char off1 = (char)data[length - 2];
      char length1 = (char)data[length - 1];
      return var5 + this.decodeLastBlock(out, var10, data1, off1, length1);
   }

   public int decode(String data, OutputStream out) {
      int var3 = 0;

      int var4;
      for(var4 = data.length(); var4 > 0 && this.ignore(data.charAt(var4 - 1)); --var4) {
      }

      int var5 = var4 - 4;

      for(int var6 = this.nextI((String)data, 0, var5); var6 < var5; var6 = this.nextI(data, var6, var5)) {
         byte var7 = this.decodingTable[data.charAt(var6++)];
         var6 = this.nextI(data, var6, var5);
         byte var8 = this.decodingTable[data.charAt(var6++)];
         var6 = this.nextI(data, var6, var5);
         byte var9;
         byte var10003 = var9 = this.decodingTable[data.charAt(var6++)];
         byte var10007 = var7;
         var6 = this.nextI(data, var6, var5);
         var7 = this.decodingTable[data.charAt(var6++)];
         out.write(var10007 << 2 | var8 >> 4);
         out.write(var8 << 4 | var9 >> 2);
         out.write(var10003 << 6 | var7);
         var3 += 3;
      }

      char var10 = data.charAt(var5);
      char data1 = data.charAt(var4 - 3);
      char out1 = data.charAt(var4 - 2);
      char var13 = data.charAt(var4 - 1);
      return var3 + this.decodeLastBlock(out, var10, data1, out1, var13);
   }
}
